#!/bin/sh
#
# Canon CAPT Printer Driver for CUPS
# Installation Script
#
#

. /etc/init.d/functions

# TODO: Сделать поиск по каталогу имени файла с возможностью
#       выбора наиболее новой версии
CNDRVCOMMON=cndrvcups-common-?.??-?.i386.rpm
CNDRVCAPT=cndrvcups-capt-?.??-?.i386.rpm

# Список поддерживаемых моделей
SUPMODS="LBP810;LBP1120;LBP1210;LBP2900;LBP3000;LBP3200;LBP3210;\
LBP3300;LBP3500;LBP3600;LBP5000;LBP5100;LBP5300"

detect_add() {

  echo -n "Поиск совместимых принтеров:";

  INFO=`lpinfo -l -v | grep -m 1 -A 4 "uri = usb://Canon" | \
        awk '{FS = "="} /info/ {print $2}' | sed 's/^ //'`;
  MODEL=`lpinfo -l -v | grep -m 1 -A 4 "uri = usb://Canon" | \
         awk '{FS = ";"} /device/ {print $2}' |\
         awk 'BEGIN {FS=":"} END {print $2}' | sed 's/.*LBP/LBP/' | \
	 sed 's/-//'`;
  
#  INFO=`cat lpinfo.log | grep -m 1 -A 4 "uri = usb://Canon" | \
#        awk '{FS = "="} /info/ {print $2}' | sed 's/^ //'`;
#  MODEL=`cat lpinfo.log | grep -m 1 -A 4 "uri = usb://Canon" | \
#         awk '{FS = ";"} /device/ {print $2}' |\
#         awk 'BEGIN {FS=":"} END {print $2}'`;

  if [[ $MODEL == "" ]]; then
    MODEL="NOTFOUND";
  fi
  
  # Проверим, поддерживается ли найденный принтер
  FIND=`echo $SUPMODS | 
        awk 'BEGIN {FS= ";"} END {for(i=1; i<=NF; ++i) print $i}' |
	grep $MODEL`;
	
  if [[ $FIND == $MODEL ]]; then
    echo_success;
    echo "";
    echo "  Обнаружены совместимые принтеры:";
    echo "    $INFO";
  else
    echo_failure;
    echo "";
    exit;
    # Не найдено поддерживаемых принтеров, поэтому выходим
  fi
			   
}

detect_del() {

  echo -n "Поиск установленных принтеров:";

  MODEL=`ccpdadmin | grep -m 1 fifo | \
       awk 'BEGIN {FS=":"} END {print $2}' | sed 's/\s//g'`;
  
#  MODEL=`cat ccpdadmin.log | grep -m 1 fifo | \
#       awk 'BEGIN {FS=":"} END {print $2}' | sed 's/\s//g'`;
  
  if [[ $MODEL != "" ]]; then
    echo_success;
    echo "";
    echo "  Обнаружены установленные принтеры:";
    echo "    $MODEL";
  else
    echo_failure;
    echo "";
  fi
}

setup() { # Установка драйверов
  echo -n "Установка драйверов CAPT:";
  if [[ -z `rpm -q cndrvcups-common 2>/dev/null` ]]; then
    # Если не установлен этот пакет, значит драйвера CAPT
    # не установлены (либо установлены с нарушенными зависимостями.
    # Поэтому устанавливаем оба пакета драйверов
    echo "INFO: Установка пакета $CNDRVCOMMON" >> setup.log;
    rpm -ihv $CNDRVCOMMON >> setup.log 2>&1;
    echo "INFO: Установка пакета $CNDRVCAPT" >> setup.log;
    rpm -ihv $CNDRVCAPT >> setup.log 2>&1;
    echo_success;
    echo "";
    service cups restart;
  elif [[ -z `rpm -q cndrvcups-capt 2>/dev/null` ]]; then
    # В случае если первый пакет установлен, то проверяем
    # установлен ли второй пакет из комплекта драйверов
    # Если установлен, то пропускаем установку драйверов!
    echo "INFO: Установка пакета $CNDRVCAPT" >> setup.log;
    rpm -ihv $CNDRVCAPT >> setup.log 2>&1;
    echo_success;
    echo "";
    service cups restart;
  else
    echo_passed;
    echo "";
    echo "WARNING: Необходимые пакеты уже установлены" >> setup.log;
  fi
  echo -n "Замена скрипта управления сервисом ccpd:";
  # Если скрипт уже отличается от того, что в пакете, то пропускаем замену
  if [[ `rpm -V cndrvcups-capt | grep /etc/init.d/ccpd` != "" ]]; then
    echo_passed;
  else
    cp -f ccpd /etc/init.d/;
    echo_success;
  fi
  chkconfig --add ccpd;
  chkconfig ccpd on;
  echo "";
}

add() { # Добавление принтеров
  if [ -e /usr/sbin/lpadmin -a -e /usr/sbin/ccpdadmin ] 
  then
    # TODO: Проверка статуса ccpd, если не запущен, то ничего не делать
    #service ccpd stop;
    if [[ $MODEL == 'LBP810' ]]; then
      MDL='LBP1120';
    else
      MDL=$MODEL;
    fi
    echo -n "Добавление принтера(ов):";
    lpadmin -p $MODEL -D "$INFO" -m CNCUPS${MDL}CAPTK.ppd -v ccp:/var/ccpd/fifo0 -E >> setup.log
    ccpdadmin -p $MODEL -o /dev/usb/lp0 >> setup.log
    echo_success;
    echo "";
    cp -f /usr/share/cups/model/CNCUPS${MDL}CAPTK.ppd /etc/cups/ppd/${MODEL}.ppd;
    service ccpd start;
  else
    echo "";
    SETCOLOR_FAILURE;
    echo -n "ERROR";
    SETCOLOR_NORMAL;
    echo ": Не установлены необходимые пакеты!";
    echo "";
    exit;
  fi
}

del() { # Удаление принтеров
  echo -n "Удаление принтера(ов):";
  echo "Удаляем принтер $MODEL из системы ..." >> setup.log;
  lpadmin -x $MODEL >> setup.log 2>&1;
  ccpdadmin -x $MODEL >> setup.log 2>&1;
  echo_success;
  echo "";
}

remove() { # Удаление драйверов
  service ccpd stop;
  echo -n "Удаление драйверов:";
  chkconfig --del ccpd;
  echo "INFO: Удаление пакета $CNDRVCAPT" >> setup.log;
  rpm -e cndrvcups-capt >> setup.log 2>&1;
  echo "INFO: Удаление пакета $CNDRVCOMMON" >> setup.log;
  rpm -e cndrvcups-common >> setup.log 2>&1;
  rm -f /etc/ccpd.conf.*
  rm -f /etc/init.d/ccpd.rpmsave
  echo_success;
  echo "";
}

if [[ `whoami` != "root" ]]; then
  echo "";
  SETCOLOR_WARNING;
  echo -n "WARNING";
  SETCOLOR_NORMAL;
  echo ": Данный скрипт необходимо выполнять с привилегиями пользователся root!";
  echo "         За подробностями обратитесь к файлу readme.txt";
  echo "";
else
  [[ -e setup.log ]] && rm -f setup.log; # Если файл лога уже существует, то удаляем
  MODEL=$2;
  case $1 in
  help) 
    help;;
  add) 
    detect_add;
    add;;
  del) 
    detect_del;
    del;
    ;;
  install) 
    setup;;
  remove) 
    detect_del;
    del;
    remove;;
  *)
    MODEL=$1;
    detect_add;
    setup;
    add;;
  esac;
fi;
